!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Program for inverting the profiles of Meteograms                 !!!
!!! Last Modified: DBS (October 29, 2019)                            !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Real, Dimension(51)	:: height, pmain, t, qv, rel_hum, u, v,&
	speed, wd, thetav, thetae, thetaes, plcl1  
	Integer, Dimension(51)	:: k

	Open (1, file = 'INPUT-PROFILE.dat', form='formatted')
	Open (2, file = 'OUTPUT-PROFILE.dat', form='formatted')
	Open (3, file = 'OUTPUT-MABL.dat', form='formatted')

	Read (1, *)

	Write (2, 10) 'JTIME', 'HEIGHT', 'KLEVEL', 'PMAIN', 'T', 'QV', &
        'RELHUM', 'U', 'V', 'WS', 'WD', 'THETAV', 'THETAE', 'THETAES', &
        'DTHETA', 'PLCL', 'PSPD', 'K', 'DVPTDZ', 'QV100', 'ZLCL' 

	Write (3, 10) 'JTIME', 'UTCTIME', 'ZLCL', 'ZMABL'
 10	Format (30(1x, a10))

	nprof = 260

	Do iprof = 1, nprof

	Do iz = 50, 1, -1
 101	Continue
	Read (1, *, err=101, end=100) xjtime, height(iz), k(iz), pmain(iz), &
        t(iz), qv(iz), rel_hum(iz), u(iz), v(iz), speed(iz), wd(iz), plcl10 

	thetav(iz)  = vpt(t(iz), rel_hum(iz), pmain(iz))
	thetae(iz)  = ept(t(iz), rel_hum(iz), pmain(iz))
	thetaes(iz) = sept(t(iz), rel_hum(iz), pmain(iz))
	plcl1(iz) = plcl(t(iz), rel_hum(iz), pmain(iz))
	Enddo

	thetav(51) = thetav(50)
	height(51) = height(50) + 1.0 ! Intentionally added 1, for avoiding 0 in denominator

	imabl = 0
	zmabl = 0.0

	Do iz = 1, 50

	dtheta = thetaes(iz) - thetae(iz)
	pspd = plcl1(iz) - pmain(iz)

	dvpt1 = thetav(iz+1) - thetav(iz)
	dz1   = height(iz+1) - height(iz)

	dvptdz = (dvpt1/dz1)*1000.0

	qv100 = (qv(iz)*100.0)/qv(1)
!	If (qv100.gt.100.0) qv100 = 100.0

	zlcl10 = zp(pmain(1),plcl1(1))

	Write (2, 15) xjtime, height(iz), k(iz), pmain(iz), t(iz), qv(iz),    &
        rel_hum(iz), u(iz), v(iz), speed(iz), wd(iz), thetav(iz), &
        thetae(iz), thetaes(iz), dtheta, plcl1(iz), pspd, iz, dvptdz, qv100, zlcl10
 15	Format (2(1x, f10.3), 1x, i10, 14(1x, f10.3), 1x, i10, 5(1x, f10.2))

	If ((imabl.eq.0).and.(dvptdz.gt.3.0)) zmabl = height(iz)
	If ((imabl.eq.0).and.(dvptdz.gt.3.0)) imabl = 1

	Enddo

	atime = (xjtime - int(xjtime))*24.0

	Write (3, 20) xjtime, atime, zlcl10, zmabl
 20	Format (1x, f10.2, 1x, f10.1, 2(1x, f10.1))

	Write (2, *) 

	Enddo        

 100	Continue

!	Stop

	End
 

 !!  Potential Temperature 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Function for estimation of potential temperature.                !!!
!!! Inputs:                                                          !!!           
!!!             temperature                     (t, deg.C)           !!!
!!!             pressure                        (p, hPa)             !!!      
!!! Output:                                                          !!!
!!!             potential temperature           (pt, deg.C)          !!!
!!! Version: February 09, 2010 (DBS).                                !!!      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        Real Function pt(t,p)

        p0      =       1000.0
        tk      =       t + 273.15
        pt      =       (tk * ((p0/p)**0.286)) - 273.15

        End Function pt

!!!  Virtual Potential Temperature 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Function for estimation of virtual potential temperature         !!!
!!! Inputs:                                                          !!!           
!!!             temperature                     (t, deg.C)           !!!
!!!             relative humidity               (rh, %)              !!!
!!!             pressure                        (p, hPa)             !!!      
!!! Output:                                                          !!!
!!!             virtual potential temperature   (vpt, deg.C)         !!!
!!! Version: February 09, 2010 (DBS).                                !!!      
!!! Version: February 09, 2010 (ANU).                                !!!      
!!! (Stull Version ?) 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        Real Function vpt(t,rh,p)

        tk      =       t + 273.15
        p0      =       1000.0
        eps     =       0.622
        esatd   =       t + 243.5
        esatn   =       17.67 * t
        esat1   =       6.112 * exp(esatn/esatd)
        e1      =       esat1 * (rh/100.0)
        r1      =       (eps*e1)/(p - e1)
        pt1     =       tk * ((p0/p)**0.286)
        vpt     =       (pt1 * (1.0 + (0.61*r1))) - 273.15

        End Function vpt

!!!  Lifting Condensation Level Pressure  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Function for estimation of pressure at LCL                       !!!
!!! Inputs:                                                          !!!           
!!!             temperature                     (t, deg.C)           !!!
!!!             relative humidity               (rh, %)              !!!
!!!             pressure                        (p, hPa)             !!!      
!!! Output:                                                          !!!
!!!             temperature at LCL  		(plcl, hPa)          !!!
!!! Version: February 09, 2010 (DBS).                                !!!      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
	Real Function plcl(t,rh,p)

	tk	=	t + 273.15
        term1d  =       1.0/(tk - 55.0)
        term2d  =       log(rh/100.0)/2840.0
        tlcl    =       55.0 + (1.0/(term1d-term2d))
        plcl    =       p * ((tlcl/tk)**3.5)

	End Function plcl

!!! Equivalent Potential Temperature 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Function for estimation of equivalent potential temperature      !!!
!!! Inputs:                                                          !!!           
!!!             temperature                     (t, deg.C)           !!!
!!!             relative humidity               (rh, %)              !!!
!!!             pressure                        (p, hPa)             !!!      
!!! Output:                                                          !!!
!!!             eq.pot.temp.                    (ept, deg.C)         !!!
!!! Version: February 09, 2010 (DBS).                                !!!      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        Real Function ept(t,rh,p)

        tk      =       t + 273.15
        If (t >= 25.0)  xlv = 2400.0       
        If (t < -40.0)  xlv = 2600.0
        xlv     =       2400.0
        p0      =       1000.0       
        cp      =       1004.0
        rd      =       287.0
        eps     =       0.622
        esatd   =       t + 243.5
        esatn   =       17.67 * t
        esat    =       6.112 * exp(esatn/esatd)
        e       =       esat * (rh/100.0)
        r       =       ((eps*e)/(p - e))
        ept1    =       tk + (xlv*r*1000.0/cp)
        ept2    =       (p0/p)**(rd/cp)
        ept     =       (ept1*ept2) - 273.15

        End Function ept

!!! Saturation Equivalent Potential Temperature 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Function for estimation of equivalent potential temperature      !!!
!!! Inputs:                                                          !!!           
!!!             temperature                     (t, deg.C)           !!!
!!!             relative humidity               (rh, %)              !!!
!!!             pressure                        (p, hPa)             !!!      
!!! Output:                                                          !!!
!!!             sat.eq.pot.temp.   (vt, deg.C)                    !!!
!!! Version: February 09, 2010 (DBS).                                !!!      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        Real Function sept(t,rh,p)

        p0      =       1000.0
        tk      =       t + 273.15
        eps     =       0.622
        esatd   =       t + 243.5
        esatn   =       17.67 * t
        esat    =       6.112 * exp(esatn/esatd)
        e       =       esat * (rh/100.0)
        rsat    =       (eps*esat)/(p - esat)
        pt      =       tk * ((p0/p)**0.286)
        sept    =       (pt * exp(2.67*1000.0*rsat/tk)) - 273.15 

        End Function sept

!!! (14) Pressure to height conversion
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Function for conversion of pressure to altitude                  !!!
!!! Inputs:                                                          !!!           
!!!             surface pressure                (p0,hPa)             !!!
!!!             pressure                        (p, hPa)             !!!      
!!! Output:                                                          !!!
!!!             Altitude                        (zp,m)               !!!
!!! Version: February 16, 2010 (ATJ).                                !!!      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
         Real Function  zp(p0,p)
 
         If (p > p0) Write (*, *) 'Value of P must be smaller than P0'
 
         term1 =         (p/p0)**0.190263
         term2 =         288.15/0.00198122
         term3 =          (1 - term1) * term2
         zp    =          term3 * 0.3048
 
         end function zp

